@@ -5,7 +5,7 @@ test_that("parameters of a constrained generated graph with 1 initial layer are
   res <- frag.simul.compare(g, layer.attr="layer", iter=60, summarise=FALSE)
   res <- frag.simul.summarise(g, layer.attr="layer", res.h1=res[[1]], res.h2=res[[2]])
   expect_equal(as.character(res$`Obs. value/H1`), 
-               c("lower", "lower", "within", "within", "within", "within", "within", "within"))
+               c("lower", "within", "within", "within", "within", "within", "within", "lower",  "lower", "lower"))
 })
 
 
